import { useCallback, useEffect, useState } from "react";
import Select from "../../components/Select";
import Slider from "../../components/Slider";
import { Tool, toolsDatabase } from "../../toolsDatabase";

const frezowaniaOpcje = {
	name: "rodzajFrezowania",
	label: "Rodzaj Frezowania",
	options: ["Czolowe", "Walcowo Czołowe", "Rowkowe"],
	images: ["/czolowe_sandvik.webp", "/walcowo-czolowe_sandvik.webp", "/rowkowe_sandvik.webp"],
};
const dropdowns = [
	{
		name: "rodzajObrobki",
		label: "Rodzaj Obróbki",
		options: ["Zgrubna", "Wykończeniowa"],
	},
	{
		name: "material",
		label: "Materiał",
		options: ["Stal węglowa", "Stal nierdzewna", "Żeliwo"],
	},
	{
		name: "tolerancja",
		label: "Klasa dokładności wykonania",
		options: ["IT6", "IT7", "IT8", "IT9", "IT10", "IT11", "IT12"],
	},
	{
		name: "chropowatosc",
		label: "Chropowatość",
		options: ["Ra 0,1", "Ra 0,2", "Ra 0,4", "Ra 0,8", "Ra 1,6", "Ra 3,2", "Ra 6,3", "Ra 12,5", "Ra 25", "Ra 50"],
	},
];
interface IInput {
	rodzajFrezowania: number;
	rodzajObrobki: number;
	material: number;
	glebokoscSkrawania: number;
	szerokoscSkrawania: number;
	tolerancja: number;
	chropowatosc: number;
}
// interface IResults {
// 	predkoscObrotowaWrzeciona: number;
// 	predkoscSkrawania: number;
// 	predkoscPosuwu: number;
// 	posowNaOstrze: number;
// 	posowNaObrot: number;
// 	wydajnoscObjetosciowa: number;
// }

const HomePage = () => {
	// ----------------------------- STATE -----------------------------
	const [input, setInput] = useState<IInput>({
		// lists
		rodzajFrezowania: 0,
		rodzajObrobki: 0,
		material: 0,
		// variables
		glebokoscSkrawania: Math.floor(Math.random() * 50 + 1),
		szerokoscSkrawania: Math.floor(Math.random() * 50 + 1),
		tolerancja: parseFloat((0.1 + Math.random() * (5 - 0.1)).toFixed(2)),
		chropowatosc: parseFloat((0.1 + Math.random() * (5 - 0.1)).toFixed(2)),
	});
	const [result, setResult] = useState<Tool | null>(null); // Znalazione narzędzie
	const [calculating, setCalculating] = useState(false);
	const [notFound, setNotFound] = useState(false);
	// ----------------------------- FUNCTION --------------------------
	const handleChangeOfField = (key: keyof typeof input) => (e: React.ChangeEvent<HTMLSelectElement>) => {
		const value = Number(e.target.value);
		setInput((prev) => ({ ...prev, [key]: value }));
	};
	const matchesInput = (tool: Tool, input: IInput): boolean => {
		return (
			tool.rodzajFrezowania === input.rodzajFrezowania &&
			tool.rodzajObrobki === input.rodzajObrobki &&
			tool.material === input.material &&
			input.chropowatosc >= tool.chropowatosc[0] &&
			input.chropowatosc <= tool.chropowatosc[1] &&
			input.szerokoscSkrawania >= tool.szerokoscSkrawania[0] &&
			input.szerokoscSkrawania <= tool.szerokoscSkrawania[1] &&
			input.glebokoscSkrawania >= tool.glebokoscSkrawania[0] &&
			input.glebokoscSkrawania <= tool.glebokoscSkrawania[1] &&
			input.tolerancja >= tool.tolerancja[0] &&
			input.tolerancja <= tool.tolerancja[1]
		);
	};
	const checkInputInDatabse = useCallback((input: IInput): void => {
		console.debug("checking...", input);

		const foundTool = toolsDatabase.tools.find((tool) => matchesInput(tool, input));

		if (foundTool) {
			console.debug("found", foundTool);
			setResult(foundTool);
		} else {
			console.debug("not found");
			setResult(null);
			setNotFound(true);
		}
	}, []);

	// ----------------------------- EFFECT ----------------------------
	useEffect(() => {
		if (calculating) {
			checkInputInDatabse(input);
		}
		setCalculating(false);
	}, [calculating, checkInputInDatabse, input]);
	useEffect(() => {
		console.dir(input);
	}, [input]);
	// ----------------------------- RENDER ----------------------------
	return (
		<>
			<div className="p-6 bg-white rounded shadow-lg lg:w-[800px] space-y-4 mb-4 mt-4 lg:ml-[50%] lg:translate-x-[-50%] ">
				<h1 className="text-3xl mb-6 text-center text-gray-700">Wybór parametrów</h1>

				<div className="mb-4">
					<label className="block text-gray-700 text-sm font-bold mb-2">{frezowaniaOpcje.label}</label>
					<div className="grid sm:grid-cols-3 grid-cols-2 gap-4 place-items-center">
						{frezowaniaOpcje.options.map((option, index) => (
							<div key={index}>
								<input
									type="radio"
									name="radio-1"
									className="radio"
									value={input.rodzajFrezowania}
									checked={input.rodzajFrezowania === index}
									onChange={() => {
										setInput((prev) => ({ ...prev, rodzajFrezowania: index }));
									}}
								/>
								<label className="text-sm ml-2 align-super">{option}</label>
								<img src={frezowaniaOpcje.images[index]} alt={option} className="max-w-[120px] mb-4 select-none" />
							</div>
						))}
					</div>
				</div>

				<div className="space-y-4">
					{dropdowns.map((dropdown) => (
						<div className="mb-4" key={dropdown.label}>
							<label className="block text-gray-700 text-sm font-bold mb-2">{dropdown.label}</label>
							<Select
								value={input[dropdown.label as keyof typeof input]}
								onChange={handleChangeOfField(dropdown.name as keyof typeof input)}
								options={dropdown.options}
								label={dropdown.label}
								disabledLabel={`Wybierz ${dropdown.label}`}
							/>
						</div>
					))}
				</div>

				<label className="flex text-gray-700 text-sm font-bold mb-0">Głębokość Skrawania</label>
				<Slider
					range={{ min: 1, max: 50 }}
					value={input.glebokoscSkrawania}
					onChange={(e) => {
						setInput((prev) => ({ ...prev, glebokoscSkrawania: Number(e!.target.value) }));
					}}
					suffix="mm"
				/>

				<label className="flex text-gray-700 text-sm font-bold mb-0">Szerokość Skrawania</label>
				<Slider
					range={{ min: 1, max: 50 }}
					value={input.szerokoscSkrawania}
					onChange={(e) => {
						setInput((prev) => ({ ...prev, szerokoscSkrawania: Number(e!.target.value) }));
					}}
					suffix="mm"
				/>

				{/* <label className="flex text-gray-700 text-sm font-bold mb-0">Klasa dokładności wykonania</label>
				<Slider
					range={{ min: 0.1, max: 5 }}
					value={input.tolerancja}
					onChange={(e) => {
						setInput((prev) => ({ ...prev, tolerancja: Number(e!.target.value) }));
					}}
					suffix="mm"
					step={0.01}
				/> */}

				{/* <label className="flex text-gray-700 text-sm font-bold mb-0">Chropowatość</label>
				<Slider
					range={{ min: 0.1, max: 5 }}
					value={input.chropowatosc}
					onChange={(e) => {
						setInput((prev) => ({ ...prev, chropowatosc: Number(e!.target.value) }));
					}}
					suffix="mm"
					step={0.01}
				/> */}

				<button
					className="btn btn-secondary w-full mt-4"
					type="submit"
					onClick={() => {
						setCalculating(true);
					}}
				>
					Dobierz narzędzia i parametry skrawania
				</button>
			</div>

			<div className="p-6 bg-white rounded shadow-lg lg:w-[500px] mb-4 lg:ml-[50%] lg:translate-x-[-50%]">
				{result ? (
					<>
						<h2 className="text-2xl mb-4">{result.name}</h2>
						<p className="text-lg">
							Płyta: <span className="text font-mono font-semibold">{result.plate}</span>
						</p>
						<p className="text-lg">
							Ilość zębów: <span className="text font-mono font-semibold">{result.ilosc_zebow}</span>
						</p>
					</>
				) : notFound ? (
					<h2 className="text-xl">❗Nie znaleziono narzędzia dla podanych parametrów</h2>
				) : (
					<p className="text-neutral-content">Tutaj pojawią się wyniki</p>
				)}
			</div>
		</>
	);
};

export default HomePage;
